@@ -12,13 +12,14 @@ https://docs.djangoproject.com/en/3.1/ref/settings/
 import os
 import re
 import logging
+import json
 
 # for printing messages before main logging config applied
 if not logging.getLogger().hasHandlers():
     logging.basicConfig(level=logging.DEBUG, format='%(message)s')
 
 from label_studio.core.utils.io import get_data_dir
-from label_studio.core.utils.params import get_bool_env, get_env
+from label_studio.core.utils.params import get_bool_env, get_env, get_env_list_int
 
 logger = logging.getLogger(__name__)
 
@@ -26,7 +27,9 @@ logger = logging.getLogger(__name__)
 HOSTNAME = get_env('HOST', '')
 if HOSTNAME:
     if not HOSTNAME.startswith('http://') and not HOSTNAME.startswith('https://'):
-        logger.info("! HOST variable found in environment, but it must start with http:// or https://, ignore it: %s", HOSTNAME)
+        logger.info(
+            "! HOST variable found in environment, but it must start with http:// or https://, ignore it: %s", HOSTNAME
+        )
         HOSTNAME = ''
     else:
         logger.info("=> Hostname correctly is set to: %s", HOSTNAME)
@@ -64,11 +67,12 @@ logger.info('=> Database and media directory: %s', BASE_DATA_DIR)
 # https://docs.djangoproject.com/en/2.1/ref/settings/#databases
 DJANGO_DB_MYSQL = 'mysql'
 DJANGO_DB_SQLITE = 'sqlite'
+DJANGO_DB_POSTGRESQL = 'postgresql'
 DJANGO_DB = 'default'
 DATABASE_NAME_DEFAULT = os.path.join(BASE_DATA_DIR, 'label_studio.sqlite3')
 DATABASE_NAME = get_env('DATABASE_NAME', DATABASE_NAME_DEFAULT)
 DATABASES_ALL = {
-    'default': {
+    DJANGO_DB_POSTGRESQL: {
         'ENGINE': 'django.db.backends.postgresql',
         'USER': get_env('POSTGRE_USER', 'postgres'),
         'PASSWORD': get_env('POSTGRE_PASSWORD', 'postgres'),
@@ -89,9 +93,10 @@ DATABASES_ALL = {
         'NAME': DATABASE_NAME,
         'OPTIONS': {
             # 'timeout': 20,
-        }
-    }
+        },
+    },
 }
+DATABASES_ALL['default'] = DATABASES_ALL[DJANGO_DB_POSTGRESQL]
 DATABASES = {'default': DATABASES_ALL.get(get_env('DJANGO_DB', 'default'))}
 
 LOGGING = {
@@ -117,19 +122,19 @@ LOGGING = {
         'console': {
             'level': get_env('LOG_LEVEL', 'WARNING'),
             'class': 'logging.StreamHandler',
-            'formatter': 'standard'
+            'formatter': 'standard',
         },
         'rq_console': {
             'level': 'WARNING',
             'class': 'rq.utils.ColorizingStreamHandler',
             'formatter': 'rq_console',
             'exclude': ['%(asctime)s'],
-        }
+        },
     },
     'root': {
         'handlers': ['console'],
         'level': get_env('LOG_LEVEL', 'WARNING'),
-    }
+    },
 }
 
 if get_bool_env('GOOGLE_LOGGING_ENABLED', False):
@@ -144,7 +149,7 @@ if get_bool_env('GOOGLE_LOGGING_ENABLED', False):
         LOGGING['handlers']['google_cloud_logging'] = {
             'level': get_env('LOG_LEVEL', 'WARNING'),
             'class': 'google.cloud.logging.handlers.CloudLoggingHandler',
-            'client': client
+            'client': client,
         }
         LOGGING['root']['handlers'].append('google_cloud_logging')
     except GoogleAuthError as e:
@@ -158,7 +163,6 @@ INSTALLED_APPS = [
     'django.contrib.messages',
     'django.contrib.staticfiles',
     'django.contrib.humanize',
-
     'drf_yasg',
     'corsheaders',
     'django_extensions',
@@ -166,18 +170,15 @@ INSTALLED_APPS = [
     'django_filters',
     'rules',
     'annoying',
-
     'rest_framework',
     'rest_framework_swagger',
     'rest_framework.authtoken',
     'drf_generators',
-
     'core',
     'users',
     'organizations',
     'data_import',
     'data_export',
-
     'projects',
     'tasks',
     'data_manager',
@@ -191,8 +192,8 @@ MIDDLEWARE = [
     'django.middleware.security.SecurityMiddleware',
     'django.contrib.sessions.middleware.SessionMiddleware',
     'django.middleware.locale.LocaleMiddleware',
-    'django.middleware.csrf.CsrfViewMiddleware',
     'core.middleware.DisableCSRF',
+    'django.middleware.csrf.CsrfViewMiddleware',
     'django.contrib.auth.middleware.AuthenticationMiddleware',
     'django.contrib.messages.middleware.MessageMiddleware',
     'django.middleware.clickjacking.XFrameOptionsMiddleware',
@@ -213,13 +214,10 @@ REST_FRAMEWORK = {
     'DEFAULT_PERMISSION_CLASSES': [
         'core.api_permissions.HasObjectPermission',
         'rest_framework.permissions.IsAuthenticated',
-
     ],
     'EXCEPTION_HANDLER': 'core.utils.common.custom_exception_handler',
-    'DEFAULT_RENDERER_CLASSES': (
-        'rest_framework.renderers.JSONRenderer',
-    ),
-    'DEFAULT_VERSIONING_CLASS': 'rest_framework.versioning.NamespaceVersioning'
+    'DEFAULT_RENDERER_CLASSES': ('rest_framework.renderers.JSONRenderer',),
+    'DEFAULT_VERSIONING_CLASS': 'rest_framework.versioning.NamespaceVersioning',
 }
 
 # CORS & Host settings
@@ -240,10 +238,7 @@ ALLOWED_HOSTS = ['*']
 
 # Auth modules
 AUTH_USER_MODEL = 'users.User'
-AUTHENTICATION_BACKENDS = [
-    'rules.permissions.ObjectPermissionBackend',
-    'django.contrib.auth.backends.ModelBackend'
-]
+AUTHENTICATION_BACKENDS = ['rules.permissions.ObjectPermissionBackend', 'django.contrib.auth.backends.ModelBackend',]
 USE_USERNAME_FOR_LOGIN = False
 
 DISABLE_SIGNUP_WITHOUT_LINK = get_bool_env('DISABLE_SIGNUP_WITHOUT_LINK', False)
@@ -270,7 +265,7 @@ TEMPLATES = [
                 'django.template.context_processors.request',
                 'django.contrib.auth.context_processors.auth',
                 'django.contrib.messages.context_processors.messages',
-                'core.context_processors.settings'
+                'core.context_processors.settings',
             ],
             'builtins': ['django.templatetags.i18n'],
         },
@@ -283,24 +278,29 @@ RQ_QUEUES = {
         'HOST': 'localhost',
         'PORT': 6379,
         'DB': 0,
-        'DEFAULT_TIMEOUT': 180
-    }
+        'DEFAULT_TIMEOUT': 180,
+    },
 }
 
 # Swagger: automatic API documentation
 SWAGGER_SETTINGS = {
     'SECURITY_DEFINITIONS': {
-        'token': {
-            'type': 'token',
-            'name': 'Token',
+        'Token': {
+            'type': 'apiKey',
+            'name': 'Authorization',
             'in': 'header',
-            'url': '/user/account'
+            'description':
+                'The token (or API key) must be passed as a request header. '
+                'You can find your user token on the User Account page in Label Studio. Example: '
+                '<br><pre><code class="language-bash">'
+                'curl https://label-studio-host/api/projects -H "Authorization: Token [your-token]"'
+                '</code></pre>'
         }
     },
     'APIS_SORTER': 'alpha',
     'SUPPORTED_SUBMIT_METHODS': ['get', 'post', 'put', 'delete', 'patch'],
-    # "DEFAULT_AUTO_SCHEMA_CLASS": "core.utils.CustomAutoSchema",
-    'OPERATIONS_SORTER': 'alpha'
+    'OPERATIONS_SORTER': 'alpha',
+
 }
 
 SENTRY_DSN = get_env('SENTRY_DSN', None)
@@ -334,12 +334,12 @@ STATIC_ROOT = os.path.join(BASE_DIR, 'static_build')
 STATICFILES_DIRS = [os.path.join(BASE_DIR, 'static')]
 STATICFILES_FINDERS = (
     'django.contrib.staticfiles.finders.FileSystemFinder',
-    'django.contrib.staticfiles.finders.AppDirectoriesFinder'
+    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
 )
 STATICFILES_STORAGE = 'core.storage.SkipMissedManifestStaticFilesStorage'
 
 # Sessions and CSRF
-SESSION_COOKIE_SECURE = bool(int(get_env('SESSION_COOKIE_SECURE', True)))
+SESSION_COOKIE_SECURE = bool(int(get_env('SESSION_COOKIE_SECURE', False)))
 CSRF_COOKIE_SECURE = bool(int(get_env('CSRF_COOKIE_SECURE', SESSION_COOKIE_SECURE)))
 CSRF_COOKIE_HTTPONLY = bool(int(get_env('CSRF_COOKIE_HTTPONLY', SESSION_COOKIE_SECURE)))
 
@@ -353,6 +353,7 @@ AVATAR_PATH = 'avatars'
 # project exports
 EXPORT_DIR = os.path.join(BASE_DATA_DIR, 'export')
 EXPORT_URL_ROOT = '/export/'
+EXPORT_MIXIN = 'data_export.mixins.ExportMixin'
 # old export dir
 os.makedirs(EXPORT_DIR, exist_ok=True)
 # dir for delayed export
@@ -374,6 +375,7 @@ EMAIL_BACKEND = get_env('EMAIL_BACKEND', 'django.core.mail.backends.dummy.EmailB
 
 ENABLE_LOCAL_FILES_STORAGE = get_bool_env('ENABLE_LOCAL_FILES_STORAGE', default=True)
 LOCAL_FILES_SERVING_ENABLED = get_bool_env('LOCAL_FILES_SERVING_ENABLED', default=False)
+LOCAL_FILES_DOCUMENT_ROOT = get_env('LOCAL_FILES_DOCUMENT_ROOT', default=os.path.abspath(os.sep))
 
 """ React Libraries: do not forget to change this dir in /etc/nginx/nginx.conf """
 # EDITOR = label-studio-frontend repository
@@ -399,24 +401,30 @@ VERSIONS_CHECK_TIME = 0
 ALLOW_ORGANIZATION_WEBHOOKS = get_bool_env('ALLOW_ORGANIZATION_WEBHOOKS', False)
 CONVERTER_DOWNLOAD_RESOURCES = get_bool_env('CONVERTER_DOWNLOAD_RESOURCES', True)
 EXPERIMENTAL_FEATURES = get_bool_env('EXPERIMENTAL_FEATURES', False)
+USE_ENFORCE_CSRF_CHECKS = get_bool_env('USE_ENFORCE_CSRF_CHECKS', True)  # False is for tests
+CLOUD_FILE_STORAGE_ENABLED = False
 
 CREATE_ORGANIZATION = 'organizations.functions.create_organization'
 GET_OBJECT_WITH_CHECK_AND_LOG = 'core.utils.get_object.get_object_with_check_and_log'
 SAVE_USER = 'users.functions.save_user'
 USER_SERIALIZER = 'users.serializers.BaseUserSerializer'
+TASK_SERIALIZER = 'tasks.serializers.BaseTaskSerializer'
+EXPORT_DATA_SERIALIZER = 'data_export.serializers.BaseExportDataSerializer'
 DATA_MANAGER_GET_ALL_COLUMNS = 'data_manager.functions.get_all_columns'
 DATA_MANAGER_ANNOTATIONS_MAP = {}
 DATA_MANAGER_ACTIONS = {}
-DATA_MANAGER_CUSTOM_FILTER_EXPRESSIONS = ''
+DATA_MANAGER_CUSTOM_FILTER_EXPRESSIONS = 'data_manager.functions.custom_filter_expressions'
+DATA_MANAGER_PREPROCESS_FILTER = 'data_manager.functions.preprocess_filter'
 USER_LOGIN_FORM = 'users.forms.LoginForm'
 PROJECT_MIXIN = 'core.mixins.DummyModelMixin'
-TASK_MIXIN = 'core.mixins.DummyModelMixin'
+TASK_MIXIN = 'tasks.mixins.TaskMixin'
 ANNOTATION_MIXIN = 'core.mixins.DummyModelMixin'
 ORGANIZATION_MIXIN = 'core.mixins.DummyModelMixin'
 USER_MIXIN = 'users.mixins.UserMixin'
 GET_STORAGE_LIST = 'io_storages.functions.get_storage_list'
-
 STORAGE_ANNOTATION_SERIALIZER = 'io_storages.serializers.StorageAnnotationSerializer'
+TASK_SERIALIZER_BULK = 'tasks.serializers.BaseTaskSerializerBulk'
+PREPROCESS_FIELD_NAME = 'data_manager.functions.preprocess_field_name'
 
 
 def project_delete(project):
@@ -436,8 +444,34 @@ USER_AUTH = user_auth
 COLLECT_VERSIONS = collect_versions_dummy
 
 WEBHOOK_TIMEOUT = float(get_env('WEBHOOK_TIMEOUT', 1.0))
+WEBHOOK_SERIALIZERS = {
+    'project': 'webhooks.serializers_for_hooks.ProjectWebhookSerializer',
+    'task': 'webhooks.serializers_for_hooks.TaskWebhookSerializer',
+    'annotation': 'webhooks.serializers_for_hooks.AnnotationWebhookSerializer',
+}
+
+EDITOR_KEYMAP = json.dumps(get_env("EDITOR_KEYMAP"))
 
 # fix a problem with Windows mimetypes for JS and PNG
 import mimetypes
+
 mimetypes.add_type("application/javascript", ".js", True)
 mimetypes.add_type("image/png", ".png", True)
+
+
+# fields name was used in DM api before
+REST_FLEX_FIELDS = {"FIELDS_PARAM": "include"}
+
+INTERPOLATE_KEY_FRAMES = get_env('INTERPOLATE_KEY_FRAMES', False)
+
+# Feature Flags
+FEATURE_FLAGS_API_KEY = get_env('FEATURE_FLAGS_API_KEY', default='any key')
+
+# we may set feature flags from file
+FEATURE_FLAGS_FROM_FILE = get_bool_env('FEATURE_FLAGS_FROM_FILE', False)
+FEATURE_FLAGS_FILE = get_env('FEATURE_FLAGS_FILE')
+# or if file is not set, default is using offline mode
+FEATURE_FLAGS_OFFLINE = get_bool_env('FEATURE_FLAGS_OFFLINE', True)
+# default value for feature flags (if not overrided by environment or client)
+FEATURE_FLAGS_DEFAULT_VALUE = False
+
