@@ -24,7 +24,7 @@ RQ_QUEUES = {}
 
 SENTRY_DSN = get_env(
     'SENTRY_DSN',
-    'https://044f6ac4baa8491f8955a7e6bd5f7f7b@o227124.ingest.sentry.io/5820521'
+    'https://68b045ab408a4d32a910d339be8591a4@o227124.ingest.sentry.io/5820521'
 )
 SENTRY_ENVIRONMENT = get_env('SENTRY_ENVIRONMENT', 'opensource')
 
@@ -33,6 +33,8 @@ FRONTEND_SENTRY_DSN = get_env(
     'https://5f51920ff82a4675a495870244869c6b@o227124.ingest.sentry.io/5838868')
 FRONTEND_SENTRY_ENVIRONMENT = get_env('FRONTEND_SENTRY_ENVIRONMENT', 'opensource')
 
+EDITOR_KEYMAP = json.dumps(get_env("EDITOR_KEYMAP"))
+
 from label_studio import __version__
 from label_studio.core.utils import sentry
 sentry.init_sentry(release_name='label-studio', release_version=__version__)
@@ -40,3 +42,6 @@ sentry.init_sentry(release_name='label-studio', release_version=__version__)
 # we should do it after sentry init
 from label_studio.core.utils.common import collect_versions
 versions = collect_versions()
+
+# in Label Studio Community version, feature flags are always ON
+FEATURE_FLAGS_DEFAULT_VALUE = True
