@@ -37,7 +37,6 @@ from drf_yasg.inspectors import CoreAPICompatInspector, NotHandled
 from collections import defaultdict
 
 from base64 import b64encode
-from lockfile import LockFile
 from datetime import datetime
 from appdirs import user_cache_dir
 from functools import wraps
@@ -275,23 +274,19 @@ def timestamp_now():
 
 
 def find_first_one_to_one_related_field_by_prefix(instance, prefix):
+    if hasattr(instance, '_find_first_one_to_one_related_field_by_prefix_cache'):
+        return getattr(instance, '_find_first_one_to_one_related_field_by_prefix_cache')
+
+    result = None
     for field in instance._meta.get_fields():
         if issubclass(type(field), models.fields.related.OneToOneRel):
             attr_name = field.get_accessor_name()
             if re.match(prefix, attr_name) and hasattr(instance, attr_name):
-                return getattr(instance, attr_name)
+                result = getattr(instance, attr_name)
+                break
 
-
-def find_first_many_to_one_related_field_by_prefix(instance, prefix):
-    '''Hard way to check if project has at least one storage'''
-
-    for field in instance._meta.get_fields():
-        if issubclass(type(field), models.fields.related.ManyToOneRel):
-            attr_name = field.get_accessor_name()
-            if re.match(prefix, attr_name) and hasattr(instance, attr_name):
-                related_instance = getattr(instance, attr_name).first()
-                if related_instance:
-                    return related_instance
+    instance._find_first_one_to_one_related_field_by_prefix_cache = result
+    return result
 
 
 def start_browser(ls_url, no_browser):
@@ -423,6 +418,7 @@ def collect_versions(force=False):
 
     # main pypi package
     result = {
+        'release': label_studio.__version__,
         'label-studio-os-package': {
             'version': label_studio.__version__,
             'short_version': '.'.join(label_studio.__version__.split('.')[:2]),
@@ -431,8 +427,7 @@ def collect_versions(force=False):
             'current_version_is_outdated': label_studio.__current_version_is_outdated__
         },
         # backend full git info
-        'label-studio-os-backend': version.get_git_commit_info(),
-        'release': label_studio.__version__
+        'label-studio-os-backend': version.get_git_commit_info()
     }
 
     # label studio frontend
@@ -609,3 +604,40 @@ def round_floats(o):
     if isinstance(o, (list, tuple)):
         return [round_floats(x) for x in o]
     return o
+
+
+class temporary_disconnect_list_signal:
+    """ Temporarily disconnect a list of signals
+        Each signal tuple: (signal_type, signal_method, object)
+        Example:
+            with temporary_disconnect_list_signal(
+                [(signals.post_delete, update_is_labeled_after_removing_annotation, Annotation)]
+                ):
+                do_something()
+    """
+    def __init__(self, signals):
+        self.signals = signals
+
+    def __enter__(self):
+        for signal in self.signals:
+            sig = signal[0]
+            receiver = signal[1]
+            sender = signal[2]
+            dispatch_uid = signal[3] if len(signal) > 3 else None
+            sig.disconnect(
+                receiver=receiver,
+                sender=sender,
+                dispatch_uid=dispatch_uid
+            )
+
+    def __exit__(self, type_, value, traceback):
+        for signal in self.signals:
+            sig = signal[0]
+            receiver = signal[1]
+            sender = signal[2]
+            dispatch_uid = signal[3] if len(signal) > 3 else None
+            sig.connect(
+                receiver=receiver,
+                sender=sender,
+                dispatch_uid=dispatch_uid
+            )
