@@ -19,7 +19,9 @@ from core.utils.common import get_object_with_check_and_log, bool_from_request,
 from projects.models import Project
 from tasks.models import Task
 from .models import DataExport, Export
-from .serializers import ExportDataSerializer, ExportSerializer, ExportCreateSerializer
+
+from .serializers import ExportDataSerializer, ExportSerializer, ExportCreateSerializer, ExportParamSerializer
+from ranged_fileresponse import RangedFileResponse
 
 logger = logging.getLogger(__name__)
 
@@ -30,6 +32,13 @@ logger = logging.getLogger(__name__)
         tags=['Export'],
         operation_summary='Get export formats',
         operation_description='Retrieve the available export formats for the current project by ID.',
+        manual_parameters=[
+            openapi.Parameter(
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this project.'),
+        ],
         responses={
             200: openapi.Response(
                 description='Export formats',
@@ -90,6 +99,12 @@ class ExportFormatsListAPI(generics.RetrieveAPIView):
                           Specify a list of task IDs to retrieve only the details for those tasks.
                           """,
             ),
+            openapi.Parameter(
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this project.'
+            ),
         ],
         tags=['Export'],
         operation_summary='Export tasks and annotations',
@@ -129,19 +144,20 @@ class ExportAPI(generics.RetrieveAPIView):
     def get_queryset(self):
         return Project.objects.filter(organization=self.request.user.active_organization)
 
+    def get_task_queryset(self, queryset):
+        return queryset
+
     def get(self, request, *args, **kwargs):
         project = self.get_object()
-        export_type = (
-            request.GET.get('exportType', 'JSON')
-            if 'exportType' in request.GET
-            else request.GET.get('export_type', 'JSON')
-        )
-        only_finished = not bool_from_request(request.GET, 'download_all_tasks', False)
+        query_serializer = ExportParamSerializer(data=request.GET)
+        query_serializer.is_valid(raise_exception=True)
+
+        export_type = query_serializer.validated_data.get('exportType') or query_serializer.validated_data['export_type']
+        only_finished = not query_serializer.validated_data['download_all_tasks']
+        download_resources = query_serializer.validated_data['download_resources']
+        interpolate_key_frames = query_serializer.validated_data['interpolate_key_frames']
+
         tasks_ids = request.GET.getlist('ids[]')
-        if 'download_resources' in request.GET:
-            download_resources = bool_from_request(request.GET, 'download_resources', True)
-        else:
-            download_resources = settings.CONVERTER_DOWNLOAD_RESOURCES
 
         logger.debug('Get tasks')
         tasks = Task.objects.filter(project=project)
@@ -157,7 +173,10 @@ class ExportAPI(generics.RetrieveAPIView):
         logger.debug('Serialize tasks for export')
         tasks = []
         for _task_ids in batch(task_ids, 1000):
-            tasks += ExportDataSerializer(query.filter(id__in=_task_ids), many=True).data
+            tasks += ExportDataSerializer(
+                self.get_task_queryset(query.filter(id__in=_task_ids)), many=True, expand=['drafts'],
+                context={'interpolate_key_frames': interpolate_key_frames}
+            ).data
         logger.debug('Prepare export files')
 
         export_stream, content_type, filename = DataExport.generate_export_file(
@@ -235,9 +254,11 @@ class ProjectExportFilesAuthCheck(APIView):
         """,
         manual_parameters=[
             openapi.Parameter(
-                name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, default=0, description='Project ID'
-            )
-        ],
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this project.')
+        ]
     ),
 )
 @method_decorator(
@@ -250,13 +271,16 @@ class ProjectExportFilesAuthCheck(APIView):
         """,
         manual_parameters=[
             openapi.Parameter(
-                name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, default=0, description='Project ID'
-            )
-        ],
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this project.')
+        ]
     ),
 )
 class ExportListAPI(generics.ListCreateAPIView):
     queryset = Export.objects.all().order_by('-created_at')
+    project_model = Project
     serializer_class = ExportSerializer
     permission_required = all_permissions.projects_change
 
@@ -270,7 +294,7 @@ class ExportListAPI(generics.ListCreateAPIView):
     def _get_project(self):
         project_pk = self.kwargs.get('pk')
         project = generics.get_object_or_404(
-            Project.objects.for_user(self.request.user),
+            self.project_model.objects.for_user(self.request.user),
             pk=project_pk,
         )
         return project
@@ -301,20 +325,20 @@ class ExportListAPI(generics.ListCreateAPIView):
         tags=['Export'],
         operation_summary='Get export by ID',
         operation_description="""
-        Retrieve information about an export file by export ID.
+        Retrieve information about an export file by export ID for a specific project.
         """,
         manual_parameters=[
             openapi.Parameter(
-                name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, default=0, description='Project ID'
-            ),
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this project.'),
             openapi.Parameter(
                 name='export_pk',
                 type=openapi.TYPE_STRING,
                 in_=openapi.IN_PATH,
-                default=0,
-                description='Export primary key',
-            ),
-        ],
+                description='Primary key identifying the export file.'),
+        ]
     ),
 )
 @method_decorator(
@@ -327,20 +351,21 @@ class ExportListAPI(generics.ListCreateAPIView):
         """,
         manual_parameters=[
             openapi.Parameter(
-                name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, default=0, description='Project ID'
-            ),
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this project.'),
             openapi.Parameter(
                 name='export_pk',
                 type=openapi.TYPE_STRING,
                 in_=openapi.IN_PATH,
-                default=0,
-                description='Export primary key',
-            ),
-        ],
+                description='Primary key identifying the export file.'),
+        ]
     ),
 )
 class ExportDetailAPI(generics.RetrieveDestroyAPIView):
     queryset = Export.objects.all()
+    project_model = Project
     serializer_class = ExportSerializer
     lookup_url_kwarg = 'export_pk'
     permission_required = all_permissions.projects_change
@@ -348,7 +373,7 @@ class ExportDetailAPI(generics.RetrieveDestroyAPIView):
     def _get_project(self):
         project_pk = self.kwargs.get('pk')
         project = generics.get_object_or_404(
-            Project.objects.for_user(self.request.user),
+            self.project_model.objects.for_user(self.request.user),
             pk=project_pk,
         )
         return project
@@ -379,20 +404,21 @@ class ExportDetailAPI(generics.RetrieveDestroyAPIView):
                 description='Selected export format',
             ),
             openapi.Parameter(
-                name='id', type=openapi.TYPE_STRING, in_=openapi.IN_PATH, default=0, description='Project ID'
-            ),
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this project.'),
             openapi.Parameter(
                 name='export_pk',
                 type=openapi.TYPE_STRING,
                 in_=openapi.IN_PATH,
-                default=0,
-                description='Export primary key',
-            ),
+                description='Primary key identifying the export file.'),
         ],
     ),
 )
 class ExportDownloadAPI(generics.RetrieveAPIView):
     queryset = Export.objects.all()
+    project_model = Project
     serializer_class = ExportSerializer
     lookup_url_kwarg = 'export_pk'
     permission_required = all_permissions.projects_change
@@ -400,7 +426,7 @@ class ExportDownloadAPI(generics.RetrieveAPIView):
     def _get_project(self):
         project_pk = self.kwargs.get('pk')
         project = generics.get_object_or_404(
-            Project.objects.for_user(self.request.user),
+            self.project_model.objects.for_user(self.request.user),
             pk=project_pk,
         )
         return project
@@ -420,10 +446,13 @@ class ExportDownloadAPI(generics.RetrieveAPIView):
             file_ = instance.file
         else:
             file_ = instance.convert_file(export_type)
+        
+        if file_ is None:
+            return HttpResponse("Can't get file", status=404)
 
         ext = file_.name.split('.')[-1]
-        response = HttpResponse(file_, content_type=f'application/{ext}')
+
+        response = RangedFileResponse(request, file_, content_type=f'application/{ext}')
         response['Content-Disposition'] = f'attachment; filename="{file_.name}"'
         response['filename'] = file_.name
         return response
-
