@@ -1,52 +1,43 @@
 """This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
 """
 import hashlib
-import io
 import logging
 import os
-import pathlib
 import shutil
 from copy import deepcopy
 from datetime import datetime
 
-import django_rq
 import ujson as json
 from core import version
-from core.label_config import parse_config
-from core.redis import redis_connected
-from core.utils.common import batch
+from core.utils.common import load_func
 from core.utils.io import get_all_files_from_dir, get_temp_dir, read_bytes_stream
 from django.conf import settings
-from django.core.cache.backends.base import default_key_func
-from django.core.files import File
-from django.db import models, transaction
+from django.db import models
+from django.db.models.signals import post_save
+from django.dispatch import receiver
 from django.utils.translation import gettext_lazy as _
-from django_rq import queues
 from label_studio_converter import Converter
-from projects.models import Project
 from tasks.models import Annotation
 
+
 logger = logging.getLogger(__name__)
 
 
-class Export(models.Model):
+ExportMixin = load_func(settings.EXPORT_MIXIN)
+
+
+class Export(ExportMixin, models.Model):
     class Status(models.TextChoices):
         CREATED = 'created', _('Created')
         IN_PROGRESS = 'in_progress', _('In progress')
         FAILED = 'failed', _('Failed')
         COMPLETED = 'completed', _('Completed')
 
-    project = models.ForeignKey(
-        'projects.Project',
-        related_name='exports',
-        on_delete=models.CASCADE,
-    )
-    created_by = models.ForeignKey(
-        settings.AUTH_USER_MODEL,
-        related_name='created_exports',
-        on_delete=models.SET_NULL,
-        null=True,
-        verbose_name=_('created by'),
+    title = models.CharField(
+        _('title'),
+        blank=True,
+        default='',
+        max_length=2048,
     )
     created_at = models.DateTimeField(
         _('created at'),
@@ -70,118 +61,38 @@ class Export(models.Model):
     )
 
     status = models.CharField(
-        _('Exporting status'),
+        _('Export status'),
         max_length=64,
         choices=Status.choices,
         default=Status.CREATED,
     )
-    counters = models.JSONField(_('Exporting meta data'), default=dict)
-
-    only_finished = models.BooleanField(
-        _('Only finished'),
-        default=False,
-        help_text=_('If true - it exports only finished tasks'),
+    counters = models.JSONField(
+        _('Exporting meta data'),
+        default=dict,
     )
-    task_ids = models.JSONField(
-        _('Task ids list'),
-        default=list,
-        help_text=_('If list is empty - download all tasks'),
+    project = models.ForeignKey(
+        'projects.Project',
+        related_name='exports',
+        on_delete=models.CASCADE,
+    )
+    created_by = models.ForeignKey(
+        settings.AUTH_USER_MODEL,
+        related_name='+',
+        on_delete=models.SET_NULL,
+        null=True,
+        verbose_name=_('created by'),
     )
 
-    def has_permission(self, user):
-        return self.project.has_permission(user)
 
-    def get_export_data(self):
-        from .serializers import ExportDataSerializer
-
-        with transaction.atomic():
-            counters = Project.objects.with_counts().filter(id=self.project.id)[0].get_counters()
-            tasks = self.project.tasks.select_related('project').prefetch_related('annotations', 'predictions')
-            if self.task_ids:
-                tasks = tasks.filter(id__in=self.task_ids)
-            if self.only_finished:
-                tasks = tasks.filter(annotations__isnull=False).distinct()
-
-            task_ids = list(tasks.values_list('id', flat=True))
-
-            logger.debug('Serialize tasks for export')
-            result = []
-            for _task_ids in batch(task_ids, 1000):
-                result += ExportDataSerializer(tasks.filter(id__in=_task_ids), many=True).data
-        return result, counters
-
-    def export_to_file(self):
-        try:
-            data, counters = self.get_export_data()
-
-            now = datetime.now()
-            json_data = json.dumps(data, ensure_ascii=False)
-            md5 = hashlib.md5(json_data.encode('utf-8')).hexdigest()
-            name = f'project-{self.project.id}-at-{now.strftime("%Y-%m-%d-%H-%M")}-{md5[0:8]}.json'
-
-            file_ = File(io.StringIO(json_data), name=name)
-            self.file.save(name, file_)
-            self.md5 = md5
-            self.counters = counters
-            self.save(update_fields=['file', 'md5', 'counters'])
-
-            self.status = self.Status.COMPLETED
-            self.save(update_fields=['status'])
-        except Exception as exc:
-            self.status = self.Status.FAILED
-            self.save(update_fields=['status'])
-            logger.exception('Export was failed')
-        finally:
-            self.finished_at = datetime.now()
-            self.save(update_fields=['finished_at'])
-
-    def run_file_exporting(self):
-        if self.status == self.Status.IN_PROGRESS:
-            logger.warning('Try to export with in progress stage')
-            return
-
-        self.status = self.Status.IN_PROGRESS
-        self.save(update_fields=['status'])
-
-        if redis_connected():
-            queue = django_rq.get_queue('default')
-            job = queue.enqueue(export_background, self.id)
-            logger.info(f'File exporting background job {job.id} for export {self} has been started')
-        else:
-            logger.info(f'Start file_exporting {self}')
-            self.export_to_file()
-
-    def convert_file(self, to):
-        with get_temp_dir() as tmp_dir:
-            converter = Converter(
-                config=self.project.get_parsed_config(),
-                project_dir=None,
-                upload_dir=tmp_dir,
-                # download_resources=download_resources,
-            )
-            input_name = pathlib.Path(self.file.name).name
-            input_file_path = pathlib.Path(tmp_dir) / input_name
-            with open(input_file_path, 'wb') as out_file:
-                out_file.write(self.file.open().read())
-
-            converter.convert(input_file_path, tmp_dir, to, is_dir=False)
-
-            files = get_all_files_from_dir(tmp_dir)
-            output_file = [file_name for file_name in files if pathlib.Path(file_name).name != input_name][0]
-
-            out = read_bytes_stream(output_file)
-            filename = pathlib.Path(input_name).stem + pathlib.Path(output_file).suffix
-            return File(
-                out,
-                name=filename,
-            )
-
-
-def export_background(export_id):
-    Export.objects.get(id=export_id).export_to_file()
+@receiver(post_save, sender=Export)
+def set_export_default_name(sender, instance, created, **kwargs):
+    if created and not instance.title:
+        instance.title = instance.get_default_title()
+        instance.save()
 
 
 class DataExport(object):
+    # TODO: deprecated
     @staticmethod
     def save_export_files(project, now, get_args, data, md5, name):
         """Generate two files: meta info and result file and store them locally for logging"""
