@@ -10,17 +10,21 @@ from rest_framework.pagination import PageNumberPagination
 from rest_framework.response import Response
 from rest_framework.views import APIView
 from drf_yasg.utils import swagger_auto_schema
+from drf_yasg import openapi
 from django.db.models import Sum, Count
+from django.conf import settings
 from ordered_set import OrderedSet
 
-from core.utils.common import get_object_with_check_and_log, int_from_request, bool_from_request, find_first_many_to_one_related_field_by_prefix
+from core.utils.common import get_object_with_check_and_log, int_from_request, bool_from_request, load_func
 from core.permissions import all_permissions, ViewClassPermission
 from projects.models import Project
 from projects.serializers import ProjectSerializer
-from tasks.models import Task, Annotation
+from tasks.models import Task, Annotation, Prediction
+from tasks.serializers import TaskIDOnlySerializer
 
-from data_manager.functions import get_all_columns, get_prepared_queryset, evaluate_predictions
-from data_manager.models import View
+from data_manager.functions import get_prepared_queryset, evaluate_predictions, get_prepare_params
+from data_manager.models import View, PrepareParams
+from data_manager.managers import get_fields_for_evaluation
 from data_manager.serializers import ViewSerializer, DataManagerTaskSerializer, SelectedItemsSerializer, ViewResetSerializer
 from data_manager.actions import get_all_actions, perform_action
 
@@ -28,34 +32,109 @@ from data_manager.actions import get_all_actions, perform_action
 logger = logging.getLogger(__name__)
 
 
+@method_decorator(name='list', decorator=swagger_auto_schema(
+    tags=['Data Manager'], operation_summary="List views",
+    operation_description="List all views for a specific project.",
+    manual_parameters=[
+        openapi.Parameter(
+            name='project',
+            type=openapi.TYPE_INTEGER,
+            in_=openapi.IN_QUERY,
+            description='Project ID'),
+    ],
+))
+@method_decorator(name='create', decorator=swagger_auto_schema(
+    tags=['Data Manager'], operation_summary="Create view",
+    operation_description="Create a view for a specific project.",
+))
+@method_decorator(name='retrieve', decorator=swagger_auto_schema(
+    tags=['Data Manager'],
+    operation_summary="Get view details",
+    operation_description="Get the details about a specific view in the data manager",
+    manual_parameters=[
+        openapi.Parameter(
+            name='id',
+            type=openapi.TYPE_STRING,
+            in_=openapi.IN_PATH,
+            description='View ID'),
+    ],
+))
+@method_decorator(name='update', decorator=swagger_auto_schema(
+    tags=['Data Manager'], operation_summary="Put view",
+    operation_description="Overwrite view data with updated filters and other information for a specific project.",
+    manual_parameters=[
+        openapi.Parameter(
+            name='id',
+            type=openapi.TYPE_STRING,
+            in_=openapi.IN_PATH,
+            description='View ID'),
+    ],
+))
+@method_decorator(name='partial_update', decorator=swagger_auto_schema(
+    tags=['Data Manager'], operation_summary="Update view",
+    operation_description="Update view data with additional filters and other information for a specific project.",
+    manual_parameters=[
+        openapi.Parameter(
+            name='id',
+            type=openapi.TYPE_STRING,
+            in_=openapi.IN_PATH,
+            description='View ID'),
+    ],
+))
+@method_decorator(name='destroy', decorator=swagger_auto_schema(
+    tags=['Data Manager'], operation_summary="Delete view",
+    operation_description="Delete a specific view by ID.",
+    manual_parameters=[
+        openapi.Parameter(
+            name='id',
+            type=openapi.TYPE_STRING,
+            in_=openapi.IN_PATH,
+            description='View ID'),
+    ],
+))
+class ViewAPI(viewsets.ModelViewSet):
+    serializer_class = ViewSerializer
+    filter_backends = [DjangoFilterBackend]
+    filterset_fields = ["project"]
+    permission_required = ViewClassPermission(
+        GET=all_permissions.tasks_view,
+        POST=all_permissions.tasks_change,
+        PATCH=all_permissions.tasks_change,
+        PUT=all_permissions.tasks_change,
+        DELETE=all_permissions.tasks_delete,
+    )
+
+    def perform_create(self, serializer):
+        serializer.save(user=self.request.user)
+
+    @swagger_auto_schema(
+        tags=['Data Manager'],
+        operation_summary="Reset project views",
+        operation_description="Reset all views for a specific project.",
+        request_body=ViewResetSerializer,
+    )
+    @action(detail=False, methods=["delete"])
+    def reset(self, request):
+        serializer = ViewResetSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        project = generics.get_object_or_404(Project.objects.for_user(request.user), pk=serializer.validated_data['project'].id)
+        queryset = self.filter_queryset(self.get_queryset()).filter(project=project)
+        queryset.all().delete()
+        return Response(status=204)
+
+    def get_queryset(self):
+        return View.objects.filter(project__organization=self.request.user.active_organization)
+
+
 class TaskPagination(PageNumberPagination):
     page_size = 100
     page_size_query_param = "page_size"
     total_annotations = 0
     total_predictions = 0
-    all_tasks = 0
-
-    def get_count(self, queryset):
-        return self.all_tasks
 
     def paginate_queryset(self, queryset, request, view=None):
-        ops = {
-            'all_tasks': Count('pk'),
-            'all_annotations': Sum("total_annotations"),
-            'all_predictions': Sum("total_predictions")
-        }
-        if 'total_predictions' not in queryset.query.annotations:
-            ops.pop('all_predictions')
-        if 'total_annotations' not in queryset.query.annotations:
-            ops.pop('all_annotations')
-
-        aggregated = queryset.aggregate(
-            **ops
-        )
-
-        self.all_tasks = aggregated['all_tasks']
-        self.total_annotations = (aggregated.get('all_annotations') or 0) if 'all_annotations' in aggregated else None
-        self.total_predictions = (aggregated.get('all_predictions') or 0) if 'all_predictions' in aggregated else None
+        self.total_predictions = Prediction.objects.filter(task_id__in=queryset).count()
+        self.total_annotations = Annotation.objects.filter(task_id__in=queryset, was_cancelled=False).count()
         return super().paginate_queryset(queryset, request, view)
 
     def get_paginated_response(self, data):
@@ -69,28 +148,27 @@ class TaskPagination(PageNumberPagination):
         )
 
 
-@method_decorator(name='list', decorator=swagger_auto_schema(
-    tags=['Data Manager'], operation_summary="List views",
-    operation_description="List all views for a specific project."))
-@method_decorator(name='create', decorator=swagger_auto_schema(
-    tags=['Data Manager'], operation_summary="Create view",
-    operation_description="Create a view for a speicfic project."))
-@method_decorator(name='retrieve', decorator=swagger_auto_schema(
-    tags=['Data Manager'], operation_summary="Get view",
-    operation_description="Get all views for a specific project."))
-@method_decorator(name='update', decorator=swagger_auto_schema(
-    tags=['Data Manager'], operation_summary="Put view",
-    operation_description="Overwrite view data with updated filters and other information for a specific project."))
-@method_decorator(name='partial_update', decorator=swagger_auto_schema(
-    tags=['Data Manager'], operation_summary="Update view",
-    operation_description="Update view data with additional filters and other information for a specific project."))
-@method_decorator(name='destroy', decorator=swagger_auto_schema(
-    tags=['Data Manager'], operation_summary="Delete view",
-    operation_description="Delete a view for a specific project."))
-class ViewAPI(viewsets.ModelViewSet):
-    serializer_class = ViewSerializer
-    filter_backends = [DjangoFilterBackend]
-    filterset_fields = ["project"]
+@method_decorator(name='get', decorator=swagger_auto_schema(
+    tags=['Data Manager'],
+    operation_summary='Get tasks list',
+    operation_description="""
+    Retrieve a list of tasks with pagination for a specific view or project, by using filters and ordering.
+    """,
+    # responses={200: DataManagerTaskSerializer(many=True)},
+    manual_parameters=[
+        openapi.Parameter(
+            name='view',
+            type=openapi.TYPE_INTEGER,
+            in_=openapi.IN_QUERY,
+            description='View ID'),
+        openapi.Parameter(
+            name='project',
+            type=openapi.TYPE_INTEGER,
+            in_=openapi.IN_QUERY,
+            description='Project ID'),
+    ],
+))
+class TaskListAPI(generics.ListAPIView):
     task_serializer_class = DataManagerTaskSerializer
     permission_required = ViewClassPermission(
         GET=all_permissions.tasks_view,
@@ -100,40 +178,12 @@ class ViewAPI(viewsets.ModelViewSet):
         DELETE=all_permissions.tasks_delete,
     )
 
-    def perform_create(self, serializer):
-        serializer.save(user=self.request.user)
-
-    @swagger_auto_schema(tags=['Data Manager'])
-    @action(detail=False, methods=['delete'])
-    def reset(self, request):
-        """
-        delete:
-        Reset project views
-
-        Reset all views for a specific project.
-        """
-        serializer = ViewResetSerializer(data=request.data)
-        serializer.is_valid(raise_exception=True)
-        project = generics.get_object_or_404(Project.objects.for_user(request.user), pk=serializer.validated_data['project'].id)
-        queryset = self.filter_queryset(self.get_queryset()).filter(project=project)
-        queryset.all().delete()
-        return Response(status=204)
-
-    def get_queryset(self):
-        return View.objects.filter(project__organization=self.request.user.active_organization)
-
     @staticmethod
     def get_task_serializer_context(request, project):
-        storage = find_first_many_to_one_related_field_by_prefix(project, '.*io_storages.*')
-        resolve_uri = True
-        if not storage and not project.task_data_login and not project.task_data_password:
-            resolve_uri = False
-
         all_fields = request.GET.get('fields', None) == 'all'  # false by default
 
         return {
-            'proxy': bool_from_request(request.GET, 'proxy', True),
-            'resolve_uri': resolve_uri,
+            'resolve_uri': True,
             'request': request,
             'project': project,
             'drafts': all_fields,
@@ -141,34 +191,55 @@ class ViewAPI(viewsets.ModelViewSet):
             'annotations': all_fields
         }
 
-    def get_task_queryset(self, request, view):
-        all_fields = 'all' if request.GET.get('fields', None) == 'all' else None
-        return Task.prepared.all(
-            fields_for_evaluation=all_fields,
-            prepare_params=view.get_prepare_tasks_params(),
-            request=request)
+    def get_task_queryset(self, request, prepare_params):
+        return Task.prepared.only_filtered(prepare_params=prepare_params)
 
-    @swagger_auto_schema(tags=['Data Manager'], responses={200: task_serializer_class(many=True)})
-    @action(detail=True, methods=["get"])
-    def tasks(self, request, pk=None):
-        """
-        get:
-        Get task list for view
+    def get(self, request):
+        # get project
+        view_pk = int_from_request(request.GET, 'view', 0) or int_from_request(request.data, 'view', 0)
+        project_pk = int_from_request(request.GET, 'project', 0) or int_from_request(request.data, 'project', 0)
+        if project_pk:
+            project = get_object_with_check_and_log(request, Project, pk=project_pk)
+            self.check_object_permissions(request, project)
+        elif view_pk:
+            view = get_object_with_check_and_log(request, View, pk=view_pk)
+            project = view.project
+            self.check_object_permissions(request, project)
+        else:
+            return Response({'detail': 'Neither project nor view id specified'}, status=404)
 
-        Retrieve a list of tasks with pagination for a specific view using filters and ordering.
-        """
-        view = self.get_object()
-        queryset = self.get_task_queryset(request, view)
-        project = view.project
+        # get prepare params (from view or from payload directly)
+        prepare_params = get_prepare_params(request, project)
+        queryset = self.get_task_queryset(request, prepare_params)
         context = self.get_task_serializer_context(self.request, project)
 
         # paginated tasks
         self.pagination_class = TaskPagination
         page = self.paginate_queryset(queryset)
+        all_fields = 'all' if request.GET.get('fields', None) == 'all' else None
+        fields_for_evaluation = get_fields_for_evaluation(prepare_params, request.user)
+
+        review = bool_from_request(self.request.GET, 'review', False)
+        if review:
+            fields_for_evaluation = ['annotators', 'reviewed']
+            all_fields = None
+
         if page is not None:
+            ids = [task.id for task in page]  # page is a list already
+            tasks = list(
+                Task.prepared.annotate_queryset(
+                    Task.objects.filter(id__in=ids),
+                    fields_for_evaluation=fields_for_evaluation,
+                    all_fields=all_fields,
+                )
+            )
+            tasks_by_ids = {task.id: task for task in tasks}
+
+            # keep ids ordering
+            page = [tasks_by_ids[_id] for _id in ids]
+
             # retrieve ML predictions if tasks don't have them
-            if project.evaluate_predictions_automatically:
-                ids = [task.id for task in page]  # page is a list already
+            if not review and project.evaluate_predictions_automatically:
                 tasks_for_predictions = Task.objects.filter(id__in=ids, predictions__isnull=True)
                 evaluate_predictions(tasks_for_predictions)
 
@@ -178,108 +249,73 @@ class ViewAPI(viewsets.ModelViewSet):
         # all tasks
         if project.evaluate_predictions_automatically:
             evaluate_predictions(queryset.filter(predictions__isnull=True))
+        queryset = Task.prepared.annotate_queryset(
+            queryset, fields_for_evaluation=fields_for_evaluation, all_fields=all_fields
+        )
         serializer = self.task_serializer_class(queryset, many=True, context=context)
         return Response(serializer.data)
 
-    @swagger_auto_schema(tags=['Data Manager'], methods=["get", "post", "delete", "patch"])
-    @action(detail=True, url_path="selected-items", methods=["get", "post", "delete", "patch"])
-    def selected_items(self, request, pk=None):
-        """
-        get:
-        Get selected items
 
-        Retrieve selected tasks for a specified view.
-
-        post:
-        Overwrite selected items
-
-        Overwrite the selected items with new data.
-
-        patch:
-        Add selected items
-
-        Add selected items to a specific view.
-
-        delete:
-        Delete selected items
-
-        Delete selected items from a specific view.
-        """
-        view = self.get_object()
-
-        # GET: get selected items from tab
-        if request.method == "GET":
-            serializer = SelectedItemsSerializer(view.selected_items)
-            return Response(serializer.data)
-
-        data = request.data
-        serializer = SelectedItemsSerializer(data=data, context={"view": view, "request": request})
-        serializer.is_valid(raise_exception=True)
-        data = serializer.validated_data
-
-        # POST: set whole
-        if request.method == "POST":
-            view.selected_items = data
-            view.save()
-            return Response(serializer.validated_data, status=201)
-
-        selected_items = view.selected_items
-        if selected_items is None:
-            selected_items = {"all": False, "included": []}
-
-        key = "excluded" if data["all"] else "included"
-        left = OrderedSet(selected_items.get(key, []))
-        right = OrderedSet(data.get(key, []))
-
-        # PATCH: set particular with union
-        if request.method == "PATCH":
-            # make union
-            result = left | right
-            view.selected_items = selected_items
-            view.selected_items[key] = list(result)
-            view.save(update_fields=["selected_items"])
-            return Response(view.selected_items, status=201)
-
-        # DELETE: delete specified items
-        if request.method == "DELETE":
-            result = left - right
-            view.selected_items[key] = list(result)
-            view.save(update_fields=["selected_items"])
-            return Response(view.selected_items, status=204)
-
-
-class TaskAPI(APIView):
+@method_decorator(name='get', decorator=swagger_auto_schema(
+    tags=['Data Manager'],
+    operation_summary='Get task by ID',
+    operation_description='Retrieve a specific task by ID.',
+    manual_parameters=[
+        openapi.Parameter(
+            name='id',
+            type=openapi.TYPE_INTEGER,
+            in_=openapi.IN_PATH,
+            description='Task ID'),
+    ],
+))
+class TaskAPI(generics.RetrieveAPIView):
     permission_required = all_permissions.projects_view
 
     def get_serializer_class(self):
         return DataManagerTaskSerializer
 
-    @staticmethod
-    def get_serializer_context(request):
+    def get_serializer_context(self, request):
+        review = bool_from_request(self.request.GET, 'review', False)
+        if review:
+            fields = ['drafts', 'annotations']
+        else:
+            fields = ['completed_by_full', 'drafts', 'predictions', 'annotations']
+
         return {
-            'proxy': bool_from_request(request.GET, 'proxy', True),
             'resolve_uri': True,
-            'completed_by': 'full',
-            'drafts': True,
-            'predictions': True,
-            'annotations': True,
+            'completed_by': 'full' if 'completed_by_full' in fields else None,
+            'predictions': 'predictions' in fields,
+            'annotations': 'annotations' in fields,
+            'drafts': 'drafts' in fields,
             'request': request
         }
 
-    @swagger_auto_schema(tags=["Data Manager"])
-    def get(self, request, pk):
-        """
-        get:
-        Task by ID
+    def get_queryset(self):
+        return Task.objects.filter(
+            project__organization=self.request.user.active_organization
+        )
 
-        Retrieve a specific task by ID.
-        """
-        task = Task.prepared.get_queryset(fields_for_evaluation='all').get(id=pk)
+    def get(self, request, pk):
+        task = self.get_object()
         context = self.get_serializer_context(request)
         context['project'] = project = task.project
 
+        review = bool_from_request(self.request.GET, 'review', False)
+        if review:
+            kwargs = {
+                'fields_for_evaluation': ['annotators', 'reviewed']
+            }
+        else:
+            kwargs = {'all_fields': True}
+
+        # we need to annotate task because before it was retrieved only for permission checks and project retrieving
+        task = Task.prepared.get_queryset(
+            prepare_params=PrepareParams(project=project.id), **kwargs
+        ).filter(id=task.id).first()
+
         # get prediction
-        if project.evaluate_predictions_automatically and not task.predictions.exists():
+        if (project.evaluate_predictions_automatically or project.show_collab_predictions) \
+                and not task.predictions.exists():
             evaluate_predictions([task])
 
         serializer = self.get_serializer_class()(task, many=False, context=context)
@@ -287,35 +323,32 @@ class TaskAPI(APIView):
         return Response(data)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+    tags=['Data Manager'],
+    operation_summary='Get data manager columns',
+    operation_description='Retrieve the data manager columns available for the tasks in a specific project.',
+))
 class ProjectColumnsAPI(APIView):
     permission_required = all_permissions.projects_view
 
-    @swagger_auto_schema(tags=["Data Manager"])
     def get(self, request):
-        """
-        get:
-        Get data manager columns
-
-        Retrieve the data manager columns available for the tasks in a specific project.
-        """
         pk = int_from_request(request.GET, "project", 1)
         project = get_object_with_check_and_log(request, Project, pk=pk)
         self.check_object_permissions(request, project)
-        data = get_all_columns(request, project)
+        GET_ALL_COLUMNS = load_func(settings.DATA_MANAGER_GET_ALL_COLUMNS)
+        data = GET_ALL_COLUMNS(project, request.user)
         return Response(data)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+    tags=['Data Manager'],
+    operation_summary='Get project state',
+    operation_description='Retrieve the project state for the data manager.',
+))
 class ProjectStateAPI(APIView):
     permission_required = all_permissions.projects_view
 
-    @swagger_auto_schema(tags=["Data Manager"])
     def get(self, request):
-        """
-        get:
-        Project state
-
-        Retrieve the project state for data manager.
-        """
         pk = int_from_request(request.GET, "project", 1)  # replace 1 to None, it's for debug only
         project = get_object_with_check_and_log(request, Project, pk=pk)
         self.check_object_permissions(request, project)
@@ -336,33 +369,29 @@ class ProjectStateAPI(APIView):
         return Response(data)
 
 
+@method_decorator(name='get', decorator=swagger_auto_schema(
+    tags=['Data Manager'],
+    operation_summary='Get actions',
+    operation_description='Retrieve all the registered actions with descriptions that data manager can use.',
+))
+@method_decorator(name='post', decorator=swagger_auto_schema(
+    tags=['Data Manager'],
+    operation_summary='Post actions',
+    operation_description='Perform an action with the selected items from a specific view.',
+))
 class ProjectActionsAPI(APIView):
     permission_required = ViewClassPermission(
         GET=all_permissions.projects_view,
         POST=all_permissions.projects_view,
     )
 
-    @swagger_auto_schema(tags=["Data Manager"])
     def get(self, request):
-        """
-        get:
-        Get actions
-
-        Retrieve all the registered actions with descriptions that data manager can use.
-        """
         pk = int_from_request(request.GET, "project", 1)  # replace 1 to None, it's for debug only
         project = get_object_with_check_and_log(request, Project, pk=pk)
         self.check_object_permissions(request, project)
-        return Response(get_all_actions(request.user))
+        return Response(get_all_actions(request.user, project))
 
-    @swagger_auto_schema(tags=["Data Manager"])
     def post(self, request):
-        """
-        post:
-        Post actions
-
-        Perform an action with the selected items from a specific view.
-        """
         pk = int_from_request(request.GET, "project", None)
         project = get_object_with_check_and_log(request, Project, pk=pk)
         self.check_object_permissions(request, project)
