@@ -6,11 +6,11 @@ import json
 import boto3
 
 from botocore.exceptions import NoCredentialsError
-from django.db import models, transaction
+from django.db import models
 from django.conf import settings
 from django.utils.translation import gettext_lazy as _
 from django.dispatch import receiver
-from django.db.models.signals import post_save
+from django.db.models.signals import post_save, post_delete, pre_delete
 
 from io_storages.base_models import ImportStorage, ImportStorageLink, ExportStorage, ExportStorageLink
 from io_storages.utils import get_uri_via_regex
@@ -21,10 +21,10 @@ from tasks.models import Annotation
 logger = logging.getLogger(__name__)
 logging.getLogger('botocore').setLevel(logging.CRITICAL)
 boto3.set_stream_logger(level=logging.INFO)
-url_scheme = 's3'
 
 clients_cache = {}
 
+
 class S3StorageMixin(models.Model):
     bucket = models.TextField(
         _('bucket'), null=True, blank=True,
@@ -66,7 +66,6 @@ class S3StorageMixin(models.Model):
         clients_cache[cache_key] = result
         return result
 
-
     def get_client(self):
         client, _ = self.get_client_and_resource()
         return client
@@ -78,13 +77,14 @@ class S3StorageMixin(models.Model):
         return client, s3.Bucket(self.bucket)
 
     def validate_connection(self, client=None):
+        logger.debug('validate_connection')
         if client is None:
             client = self.get_client()
         if self.prefix:
             logger.debug(f'Test connection to bucket {self.bucket} with prefix {self.prefix}')
             result = client.list_objects_v2(Bucket=self.bucket, Prefix=self.prefix, MaxKeys=1)
             if not result.get('KeyCount'):
-                raise KeyError(f's3://{self.bucket}/{self.prefix} not found.')
+                raise KeyError(f'{self.url_scheme}://{self.bucket}/{self.prefix} not found.')
         else:
             logger.debug(f'Test connection to bucket {self.bucket}')
             client.head_bucket(Bucket=self.bucket)
@@ -92,7 +92,7 @@ class S3StorageMixin(models.Model):
     @property
     def path_full(self):
         prefix = self.prefix or ''
-        return f'{url_scheme}://{self.bucket}/{prefix}'
+        return f'{self.url_scheme}://{self.bucket}/{prefix}'
 
     @property
     def type_full(self):
@@ -104,6 +104,8 @@ class S3StorageMixin(models.Model):
 
 class S3ImportStorage(S3StorageMixin, ImportStorage):
 
+    url_scheme = 's3'
+
     presign = models.BooleanField(
         _('presign'), default=True,
         help_text='Generate presigned URLs')
@@ -146,7 +148,7 @@ class S3ImportStorage(S3StorageMixin, ImportStorage):
         return parsed_data
 
     def get_data(self, key):
-        uri = f's3://{self.bucket}/{key}'
+        uri = f'{self.url_scheme}://{self.bucket}/{key}'
         if self.use_blob_urls:
             data_key = settings.DATA_UNDEFINED_NAME
             return {data_key: uri}
@@ -162,15 +164,8 @@ class S3ImportStorage(S3StorageMixin, ImportStorage):
         value = self._get_validated_task(value, key)
         return value
 
-    def resolve_uri(self, data):
-        try:
-            uri, storage = get_uri_via_regex(data, prefixes=(url_scheme,))
-            if not storage:
-                return
-            resolved_uri = resolve_s3_url(uri, self.get_client(), self.presign, expires_in=self.presign_ttl * 60)
-            return data.replace(uri, resolved_uri)
-        except NoCredentialsError:
-            logger.warning(f'No AWS credentials specified for {data}')
+    def generate_http_url(self, url):
+        return resolve_s3_url(url, self.get_client(), self.presign, expires_in=self.presign_ttl * 60)
 
 
 class S3ExportStorage(S3StorageMixin, ExportStorage):
@@ -190,6 +185,20 @@ class S3ExportStorage(S3StorageMixin, ExportStorage):
         # create link if everything ok
         S3ExportStorageLink.create(annotation, self)
 
+    def delete_annotation(self, annotation):
+        client, s3 = self.get_client_and_resource()
+        logger.debug(f'Deleting object on {self.__class__.__name__} Storage {self} for annotation {annotation}')
+
+        # get key that identifies this object in storage
+        key = S3ExportStorageLink.get_key(annotation)
+        key = str(self.prefix) + '/' + key if self.prefix else key
+
+        # delete object from storage
+        s3.Object(self.bucket, key).delete()
+
+        # delete link if everything ok
+        S3ExportStorageLink.objects.filter(storage=self, annotation=annotation).delete()
+
 
 @receiver(post_save, sender=Annotation)
 def export_annotation_to_s3_storages(sender, instance, **kwargs):
@@ -200,6 +209,16 @@ def export_annotation_to_s3_storages(sender, instance, **kwargs):
             storage.save_annotation(instance)
 
 
+@receiver(pre_delete, sender=Annotation)
+def delete_annotation_from_s3_storages(sender, instance, **kwargs):
+    links = S3ExportStorageLink.objects.filter(annotation=instance)
+    for link in links:
+        storage = link.storage
+        if storage.can_delete_objects:
+            logger.debug(f'Delete {instance} from S3 storage {storage}')
+            storage.delete_annotation(instance)
+
+
 class S3ImportStorageLink(ImportStorageLink):
     storage = models.ForeignKey(S3ImportStorage, on_delete=models.CASCADE, related_name='links')
 
