@@ -10,7 +10,8 @@ if __name__ == '__main__':
     try:
         from django.core.management import execute_from_command_line
         from django.core.management.commands.runserver import Command as runserver
-        runserver.default_port = '8080'
+        from django.conf import settings
+        runserver.default_port = settings.INTERNAL_PORT
 
     except ImportError as exc:
         raise ImportError(
