@@ -37,6 +37,19 @@ logger = logging.getLogger(__name__)
     """.format(
             host=(settings.HOSTNAME or 'https://localhost:8080')
         ),
+        request_body=openapi.Schema(
+            type=openapi.TYPE_OBJECT,
+            properties={
+                'project': openapi.Schema(
+                    type=openapi.TYPE_INTEGER,
+                    description='Project ID'
+                ),
+                'url': openapi.Schema(
+                    type=openapi.TYPE_STRING,
+                    description='ML backend URL'
+                ),
+            },
+        ),
     ),
 )
 @method_decorator(
@@ -52,6 +65,13 @@ logger = logging.getLogger(__name__)
     """.format(
             host=(settings.HOSTNAME or 'https://localhost:8080')
         ),
+        manual_parameters=[
+            openapi.Parameter(
+                name='project',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_QUERY,
+                description='Project ID'),
+        ],
     ))
 class MLBackendListAPI(generics.ListCreateAPIView):
     parser_classes = (JSONParser, FormParser, MultiPartParser)
@@ -150,6 +170,13 @@ class MLBackendDetailAPI(generics.RetrieveUpdateDestroyAPIView):
         
         Get the ML backend ID by [listing the ML backends for a project](https://labelstud.io/api/#operation/api_ml_list).
         """,
+        manual_parameters=[
+            openapi.Parameter(
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this ML backend.'),
+        ],
         request_body=openapi.Schema(
             type=openapi.TYPE_OBJECT,
             properties={
@@ -192,8 +219,15 @@ class MLBackendTrainAPI(APIView):
         operation_description="""
         Send a request to the machine learning backend set up to be used for interactive preannotations to retrieve a
         predicted region based on annotator input. 
-        See [set up machine learning](ml.html#Get-interactive-preannotations) for more.
+        See [set up machine learning](labelstud.io/guide/ml.html#Get-interactive-preannotations) for more.
         """,
+        manual_parameters=[
+            openapi.Parameter(
+                name='id',
+                type=openapi.TYPE_INTEGER,
+                in_=openapi.IN_PATH,
+                description='A unique integer value identifying this ML backend.'),
+        ],
         request_body=MLInteractiveAnnotatingRequest,
         responses={
             200: openapi.Response(title='Annotating OK', description='Interactive annotation has succeeded.'),
@@ -201,6 +235,9 @@ class MLBackendTrainAPI(APIView):
     ),
 )
 class MLBackendInteractiveAnnotating(APIView):
+
+    permission_required = all_permissions.tasks_view
+
     def post(self, request, *args, **kwargs):
         ml_backend = get_object_with_check_and_log(request, MLBackend, pk=self.kwargs['pk'])
         self.check_object_permissions(self.request, ml_backend)
