@@ -25,6 +25,9 @@ _api_urlpattens = [
 
 # TODO: these urlpatterns should be moved in core/urls with include('organizations.urls')
 urlpatterns = [
+    path('organization/', views.simple_view, name='organization-simple'),
+    path('organization/webhooks', views.simple_view, name='organization-simple'),
+
     path('people/', include(_urlpatterns)),
     path('api/organizations/', include((_api_urlpattens, app_name), namespace='api')),
 
