@@ -2,6 +2,7 @@
 """
 import logging
 import ujson as json
+import numbers
 
 from django.db import transaction
 from drf_dynamic_fields import DynamicFieldsMixin
@@ -12,6 +13,7 @@ from rest_framework.serializers import ModelSerializer
 from rest_framework.exceptions import ValidationError
 from rest_framework.fields import SkipField
 from rest_framework.settings import api_settings
+from rest_flex_fields import FlexFieldsModelSerializer
 
 from projects.models import Project
 from tasks.models import Task, Annotation, AnnotationDraft, Prediction
@@ -19,6 +21,7 @@ from tasks.validation import TaskValidator
 from core.utils.common import get_object_with_check_and_log, retry_database_locked
 from core.label_config import replace_task_data_undefined_with_config_field
 from users.serializers import UserSerializer
+from core.utils.common import load_func
 
 logger = logging.getLogger(__name__)
 
@@ -29,7 +32,7 @@ class PredictionQuerySerializer(serializers.Serializer):
 
 
 class PredictionSerializer(ModelSerializer):
-    model_version = serializers.CharField(allow_blank=True)
+    model_version = serializers.CharField(allow_blank=True, required=False)
     created_ago = serializers.CharField(default='', read_only=True, help_text='Delta time from creation time')
 
     class Meta:
@@ -41,19 +44,23 @@ class ListAnnotationSerializer(serializers.ListSerializer):
     pass
 
 
-class AnnotationSerializer(DynamicFieldsMixin, ModelSerializer):
+class AnnotationSerializer(ModelSerializer):
     """
     """
     created_username = serializers.SerializerMethodField(default='', read_only=True, help_text='Username string')
     created_ago = serializers.CharField(default='', read_only=True, help_text='Time delta from creation time')
-    completed_by = serializers.SerializerMethodField(default='Requesting user',
-                                                     help_text='ID of the user who completed the annotation.')
 
     @classmethod
     def many_init(cls, *args, **kwargs):
         kwargs['child'] = cls(*args, **kwargs)
         return ListAnnotationSerializer(*args, **kwargs)
 
+    def to_representation(self, instance):
+        annotation = super(AnnotationSerializer, self).to_representation(instance)
+        if self.context.get('completed_by', '') == 'full':
+            annotation['completed_by'] = UserSerializer(instance.completed_by).data
+        return annotation
+
     def get_fields(self):
         fields = super(AnnotationSerializer, self).get_fields()
         excluded = []
@@ -93,12 +100,6 @@ class AnnotationSerializer(DynamicFieldsMixin, ModelSerializer):
         name += f' {user.email}, {user.id}'
         return name
 
-    def get_completed_by(self, annotation):
-        if self.context.get('completed_by', '') == 'full':
-            return UserSerializer(annotation.completed_by).data
-        else:
-            return annotation.completed_by.id if annotation.completed_by else None
-
     class Meta:
         model = Annotation
         exclude = ['prediction', 'result_count']
@@ -125,7 +126,7 @@ class TaskSimpleSerializer(ModelSerializer):
         fields = '__all__'
 
 
-class TaskSerializer(ModelSerializer):
+class BaseTaskSerializer(FlexFieldsModelSerializer):
     """ Task Serializer with project scheme configs validation
     """
     def __init__(self, *args, **kwargs):
@@ -159,7 +160,7 @@ class TaskSerializer(ModelSerializer):
         if project:
             # resolve uri for storage (s3/gcs/etc)
             if self.context.get('resolve_uri', False):
-                instance.data = instance.resolve_uri(instance.data, proxy=self.context.get('proxy', False))
+                instance.data = instance.resolve_uri(instance.data, project)
 
             # resolve $undefined$ key in task data
             data = instance.data
@@ -172,12 +173,16 @@ class TaskSerializer(ModelSerializer):
         fields = '__all__'
 
 
-class TaskSerializerBulk(serializers.ListSerializer):
+class BaseTaskSerializerBulk(serializers.ListSerializer):
     """ Serialize task with annotation from source json data
     """
     annotations = AnnotationSerializer(many=True, default=[], read_only=True)
     predictions = PredictionSerializer(many=True, default=[], read_only=True)
 
+    @property
+    def project(self):
+        return self.context.get('project')
+
     @staticmethod
     def format_error(i, detail, item):
         if len(detail) == 1:
@@ -265,10 +270,9 @@ class TaskSerializerBulk(serializers.ListSerializer):
         db_tasks, db_annotations, db_predictions, validated_tasks = [], [], [], validated_data
         logging.info(f'Try to serialize tasks with annotations, data len = {len(validated_data)}')
         user = self.context.get('user', None)
-        project = self.context.get('project')
 
         organization = user.active_organization \
-            if not project.created_by.active_organization else project.created_by.active_organization
+            if not self.project.created_by.active_organization else self.project.created_by.active_organization
         members_email_to_id = dict(organization.members.values_list('user__email', 'user__id'))
         members_ids = set(members_email_to_id.values())
         logger.debug(f"{len(members_email_to_id)} members found in organization {organization}")
@@ -282,21 +286,22 @@ class TaskSerializerBulk(serializers.ListSerializer):
                 annotations = task.pop('annotations', [])
                 # insert a valid "completed_by_id" by existing member
                 self._insert_valid_completed_by_id_or_raise(
-                    annotations, members_email_to_id, members_ids, user or project.created_by)
+                    annotations, members_email_to_id, members_ids, user or self.project.created_by)
                 predictions = task.pop('predictions', [])
                 task_annotations.append(annotations)
                 task_predictions.append(predictions)
 
             # add tasks first
-            for task in validated_tasks:
-                t = Task(project=project, data=task['data'], meta=task.get('meta', {}),
-                         overlap=project.maximum_annotations,
+            max_overlap = self.project.maximum_annotations
+            for i, task in enumerate(validated_tasks):
+                t = Task(project=self.project, data=task['data'], meta=task.get('meta', {}),
+                         overlap=max_overlap, is_labeled=len(task_annotations[i]) >= max_overlap,
                          file_upload_id=task.get('file_upload_id'))
                 db_tasks.append(t)
 
-            # deprecated meta warning
-            if 'meta' in task:
-                logger.warning('You task data has field "meta" which is deprecated and it will be removed in future')
+                # deprecated meta warning
+                if 'meta' in task:
+                    logger.warning('You task data has field "meta" which is deprecated and it will be removed in future')
 
             if settings.DJANGO_DB == settings.DJANGO_DB_SQLITE:
                 self.db_tasks = []
@@ -317,20 +322,30 @@ class TaskSerializerBulk(serializers.ListSerializer):
             # add annotations
             for i, annotations in enumerate(task_annotations):
                 for annotation in annotations:
+                    if not isinstance(annotation, dict):
+                        continue
+                        
                     # support both "ground_truth" and "ground_truth"
                     ground_truth = annotation.pop('ground_truth', True)
                     was_cancelled = annotation.pop('was_cancelled', False)
+                    lead_time = annotation.pop('lead_time', None)
 
                     db_annotations.append(Annotation(task=self.db_tasks[i],
                                                      ground_truth=ground_truth,
                                                      was_cancelled=was_cancelled,
                                                      completed_by_id=annotation['completed_by_id'],
-                                                     result=annotation['result']))
+                                                     result=annotation['result'],
+                                                     lead_time=lead_time))
 
             # add predictions
             last_model_version = None
             for i, predictions in enumerate(task_predictions):
                 for prediction in predictions:
+                    if not isinstance(prediction, dict):
+                        continue
+
+                    # we need to call result normalizer here since "bulk_create" doesn't call save() method
+                    result = Prediction.prepare_prediction_result(prediction['result'], self.project)
                     prediction_score = prediction.get('score')
                     if prediction_score is not None:
                         try:
@@ -343,7 +358,7 @@ class TaskSerializerBulk(serializers.ListSerializer):
 
                     last_model_version = prediction.get('model_version', 'undefined')
                     db_predictions.append(Prediction(task=self.db_tasks[i],
-                                                     result=prediction['result'],
+                                                     result=result,
                                                      score=prediction_score,
                                                      model_version=last_model_version))
 
@@ -369,17 +384,25 @@ class TaskSerializerBulk(serializers.ListSerializer):
             logging.info(f'Predictions serialization success, len = {len(self.db_predictions)}')
 
             # renew project model version if it's empty
-            if not project.model_version and last_model_version is not None:
-                project.model_version = last_model_version
-                project.save()
+            if not self.project.model_version and last_model_version is not None:
+                self.project.model_version = last_model_version
+                self.project.save()
 
+        self.post_process_annotations(self.db_annotations)
         return db_tasks
 
+    @staticmethod
+    def post_process_annotations(db_annotations):
+        pass
+
     class Meta:
         model = Task
         fields = "__all__"
     
+
+TaskSerializer = load_func(settings.TASK_SERIALIZER)
         
+
 class TaskWithAnnotationsSerializer(TaskSerializer):
     """
     """
@@ -389,7 +412,8 @@ class TaskWithAnnotationsSerializer(TaskSerializer):
 
     class Meta:
         model = Task
-        list_serializer_class = TaskSerializerBulk
+        list_serializer_class = load_func(settings.TASK_SERIALIZER_BULK)
+        
         exclude = ()
 
 
@@ -506,6 +530,27 @@ class TaskWithAnnotationsAndPredictionsAndDraftsSerializer(TaskSerializer):
         return AnnotationDraftSerializer(drafts, many=True, read_only=True, default=[], context=self.context).data
 
 
+class NextTaskSerializer(TaskWithAnnotationsAndPredictionsAndDraftsSerializer):
+    def get_predictions(self, task):
+        project = task.project
+        if not project.show_collab_predictions:
+            return []
+        else:
+            for ml_backend in project.ml_backends.all():
+                ml_backend.predict_tasks([task])
+            return super().get_predictions(task)
+
+    def get_annotations(self, task):
+        result = []
+        annotations = super().get_annotations(task)
+        if 'request' in self.context and hasattr(self.context['request'], 'user'):
+            user = self.context['request'].user
+            for annotation in annotations:
+                if annotation.get('completed_by') == user.id and not (annotation.get('ground_truth') or annotation.get('honeypot')):
+                    result.append(annotation)
+        return result
+
+
 class TaskIDWithAnnotationsAndPredictionsSerializer(ModelSerializer):
 
     def __init__(self, *args, **kwargs):
@@ -516,3 +561,14 @@ class TaskIDWithAnnotationsAndPredictionsSerializer(ModelSerializer):
     class Meta:
         model = Task
         fields = ['id', 'annotations', 'predictions']
+
+
+class TaskIDOnlySerializer(ModelSerializer):
+
+    class Meta:
+        model = Task
+        fields = ['id']
+
+
+# LSE inherits this serializer
+TaskSerializerBulk = load_func(settings.TASK_SERIALIZER_BULK)
