@@ -3,9 +3,8 @@
 import pytest
 import json
 
-from ..utils import make_task, make_annotation, make_prediction, project_id
+from ..utils import make_task, make_annotation, make_prediction, project_id, make_annotator
 from projects.models import Project
-from data_manager.models import View
 from data_import.models import FileUpload
 from django.conf import settings
 from django.core.files.base import ContentFile
@@ -77,7 +76,7 @@ def test_views_ordering(ordering, element_index, undefined, business_client, pro
 
     task_ids = [task_id_1, task_id_2]
 
-    response = business_client.get(f"/api/dm/views/{view_id}/tasks/")
+    response = business_client.get(f"/api/dm/tasks?view={view_id}")
     response_data = response.json()
 
     assert response_data["tasks"][0]["id"] == task_ids[element_index]
@@ -284,10 +283,33 @@ def test_views_ordering(ordering, element_index, undefined, business_client, pro
             },
             [2, 3, 4],
         ],
+        [
+            {
+                "conjunction": "and",
+                "items": [
+                    {"filter": "filter:tasks:annotators", "operator": "contains", "value": "$ANN1_ID", "type": "List"},
+                    {"filter": "filter:tasks:annotators", "operator": "contains", "value": "$ANN2_ID", "type": "List"},
+                ],
+            },
+            [2],
+        ],
     ],
 )
 @pytest.mark.django_db
 def test_views_filters(filters, ids, business_client, project_id):
+    project = Project.objects.get(pk=project_id)
+    ann1 = make_annotator({'email': 'ann1@testheartex.com'}, project)
+    ann2 = make_annotator({'email': 'ann2@testheartex.com'}, project)
+
+    ann_ids = {
+        '$ANN1_ID': ann1.id,
+        '$ANN2_ID': ann2.id,
+    }
+    for item in filters['items']:
+        for ann_id_key, ann_id_value in ann_ids.items():
+            if isinstance(item['value'], str) and ann_id_key in item['value']:
+                item['value'] = ann_id_value
+
     payload = dict(
         project=project_id,
         data={"test": 1, "filters": filters},
@@ -301,17 +323,15 @@ def test_views_filters(filters, ids, business_client, project_id):
     assert response.status_code == 201, response.content
     view_id = response.json()["id"]
 
-    project = Project.objects.get(pk=project_id)
-
     task_data_field_name = settings.DATA_UNDEFINED_NAME
 
     task_id_1 = make_task({"data": {task_data_field_name: "some text1"}}, project).id
-    make_annotation({"result": [{"from_name": "1_first", "to_name": "", "value": {}}]}, task_id_1)
+    make_annotation({"result": [{"from_name": "1_first", "to_name": "", "value": {}}], "completed_by": ann1}, task_id_1)
     make_prediction({"result": [{"from_name": "1_first", "to_name": "", "value": {}}], "score": 1}, task_id_1)
 
     task_id_2 = make_task({"data": {task_data_field_name: "some text2"}}, project).id
-    for _ in range(0, 2):
-        make_annotation({"result": [{"from_name": "2_second", "to_name": "", "value": {}}], "was_cancelled": True}, task_id_2)
+    for ann in (ann1, ann2):
+        make_annotation({"result": [{"from_name": "2_second", "to_name": "", "value": {}}], "was_cancelled": True, "completed_by": ann}, task_id_2)
     for _ in range(0, 2):
         make_prediction({"result": [{"from_name": "2_second", "to_name": "", "value": {}}], "score": 2}, task_id_2)
 
@@ -338,7 +358,7 @@ def test_views_filters(filters, ids, business_client, project_id):
         content_type="application/json",
     )
 
-    response = business_client.get(f"/api/dm/views/{view_id}/tasks/")
+    response = business_client.get(f"/api/dm/tasks/?view={view_id}")
     response_data = response.json()
 
     assert 'tasks' in response_data, response_data
