@@ -4,9 +4,10 @@ from rest_framework import serializers
 from django.conf import settings
 
 from .models import User
+from core.utils.common import load_func
 
 
-class UserSerializer(serializers.ModelSerializer):
+class BaseUserSerializer(serializers.ModelSerializer):
     # short form for user presentation
     initials = serializers.SerializerMethodField(default='?', read_only=True)
     avatar = serializers.SerializerMethodField(read_only=True)
@@ -33,7 +34,10 @@ class UserSerializer(serializers.ModelSerializer):
         )
 
 
-class UserSimpleSerializer(UserSerializer):
+class UserSimpleSerializer(BaseUserSerializer):
     class Meta:
         model = User
         fields = ('id', 'first_name', 'last_name', 'email', 'avatar')
+
+
+UserSerializer = load_func(settings.USER_SERIALIZER)
